;*******************************************************************************
;*
;*	Titel:	Bibliothek mit Zeitunterprogrammen (SR_TIME_8M.asm)
;*	Datum:	10/05/07	Version:	0.2	
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*	Prozessor:	ATMEGA32	Quarzfrequenz:	 extern 8MHz
;*			
;*	Informationen zur Funktionsweise:
;*
;*	Die Bibliothek enthaelt Zeitschleifenunterprogramme zur Verwendung mit
;*	einem externen 8 MHz Quarz.
;*			
;*******************************************************************************

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 1ms (Wait 1ms)
;-------------------------------------------------------------------------------
W1ms:   push    WL      ;rette verwendete Registerinhalte
        push    WH
        ldi     WL,0xCC ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x07 ;G = (t-tT)/4tT = 1ms/4*125ns-1/4 = 2000-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 2000-4 = 1996 = 0x07CC
W1msL:  sbiw    WL,1    ;Dekrementiere den Schleifenzaehler W
        brne    W1msL   ;Verlasse die Schleife bei Schleifenzaehler W=0
        pop     WH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     WL
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 1ms (Wait X*1ms)
;	Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;	Der Fehler durch die zusaetzlichen Befehle kann hier vernachlässigt
;	werden (max Fehler bei einmaligem Aufruf < 0,12%).
;-------------------------------------------------------------------------------
WX1ms:	push	XL	;rette verwendete Registerinhalte
	push	XH
WX1msL:	rcall	W1MS	;Rufe innere Schleife (Unterprogramm W1ms)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	WX1msL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 5ms (Wait 5ms)
;       Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------
W5MS:   push    r24     ;rette verwendete Registerinhalte
        ldi     r24,5   ;Zaehler = 5
W5MSL:  rcall   W1MS    ;Rufe innere Schleife (Unterprogramm W1ms)
        dec     r24     ;Dekrementiere den Schleifenzaehler W
        brne    W5MSL ;Verlasse die Schleife bei Schleifenzaehler W = 0
        pop     r24     ;Wiederherstellung der verwendeten Registerinhalte
        ret             ;Ruecksprung ins Hauptprogramm
        
;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 100ms (Wait 100ms)
;       Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------
W100MS: push    r24     ;rette verwendete Registerinhalte
        ldi     r24,100 ;Zaehler = 100
W100MSL: rcall  W1MS    ;Rufe innere Schleife (Unterprogramm W1ms)
        dec     r24     ;Dekrementiere den Schleifenzaehler W
        brne    W100MSL ;Verlasse die Schleife bei Schleifenzaehler W = 0
        pop     r24     ;Wiederherstellung der verwendeten Registerinhalte
        ret             ;Ruecksprung ins Hauptprogramm
        
;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 5s (Wait 5s)
;       Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------
W5s:    push    r24     ;rette verwendete Registerinhalte
        push    r25
        ldi     r24,low(5000)
        ldi     r25,high(5000)
W5sL:   rcall   W1ms    ;Rufe innere Schleife (Unterprogramm W1ms)
        sbiw    r24,1   ;Dekrementiere den Schleifenzaehler W
        brne    W5sL    ;Verlasse die Schleife bei Schleifenzaehler W = 0
        pop     r25     ;Wiederherstellung der verwendeten Registerinhalte
        pop     r24     
        ret             ;Ruecksprung ins Hauptprogramm

