;*******************************************************************************
;*
;*      Titel:  Kleine USB-Bibliothek:
;*              Unterprogramme und Interruptroutinen zur Nutzung von USB mit
;*              ATMEL-AVR-ICs.
;*              HID Klasse (SR_USB_HID_V1_3.asm)
;*	Datum:	11/03/12	Version:	1.3 (07/04/16)
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*      
;*      Controller: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u2  
;*                  (siehe Zuweisungen im Hauptprogramm)
;*                 
;*      Quarz:      8 oder 16 MHz  (siehe Zuweisungen im Hauptprogramm)
;*                 
;*      Informationen zur Funktionsweise:
;*      
;*      In den Zuweisungen im Hauptprogramm wird festgelegt welcher Chip genutzt
;*      wird und mit welcher Quarzfrequenz der Chip getaktet wird.
;*      Auch kann hier ausgewaehlt werden ob eine Maus bzw. eine
;*      Tastatur-Emulation erwuenscht ist.
;*      
;*      Im Hauptprogramm müssen die beiden folgenden Interruptvektoren
;*      initialisiert sein: 
;*      
;*      .ORG    USB_GENaddr       ;interner Vektor fuer USB General Interrupt
;*              rjmp    ISRGEN    ;Springe zur ISR des GENeral Interrupt
;*      .ORG    USB_COMaddr       ;interner Vektor fuer USB Endpoint/Pipe
;*                                ;COMmunication Interrupt
;*              rjmp    ISRCOM    ;Springe zur ISR des EP COMmunication Interrupt
;*              
;*      Die Konstante CRYSTAL muss definiert sein. Mit 
;*      .INCLUDE "lib/SR_USB_HID_V1_2.asm" wird die USB Bibliothek eingebunden.
;*      Durch den Aufruf eines Unterprogramms (rcall USBINI) wird diese 
;*      initialisiert.
;*              
;*      Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
;*      abgewickelt. Alle dazu benötigten USB-Routinen und Unterprogramme
;*      befinden sich in dieser USB-Bibliothek.
;*      
;*      Fuer beide Endpunkte steht ein 64-Byte großer Buffer zur Verfuegung. 
;*      
;*      Beim Endpunkt 1 (IN) fragt der PC Daten an.
;*      Durch Setzen eines Flags (EP1_FLAG) im Hauptprogramm wird dem Endpunkt
;*      mitgeteilt, wenn neue Daten vorliegen. Die Anzahl der vorhandenen Bytes
;*      steht dann in der Variablen EP1_CNT. Die Daten selbst werden im Buffer
;*      (EP1_BUF) abgelegt.
;*      
;*      Ueber den Endpunkt 2 (OUT) sendet der PC Daten.
;*      Die Hauptschleife im Hauptprogramm erkennt an einem Flag (EP2_FLAG)
;*      wenn Daten vorliegen.
;*      Die Daten liegen im Buffer (EP2_BUF). Die Variable EP2_CNT gibt an wie 
;*      viele Bytes gesendet wurden.
;*      
;*      Weitere Dokumentation auf www.weigu.lu/b/usb
;*      
;*      Copyright (c) 2012   Guy WEILER         weigu[at]weigu[dot]lu
;*      
;*      This program is free software: you can redistribute it and/or modify
;*      it under the terms of the GNU General Public License as published by
;*      the Free Software Foundation, either version 3 of the License, or
;*      (at your option) any later version.
;*
;*      This program is distributed in the hope that it will be useful,
;*      but WITHOUT ANY WARRANTY; without even the implied warranty of
;*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;*      GNU General Public License for more details.
;*
;*      You should have received a copy of the GNU General Public License
;*      along with this program.  If not, see <http://www.gnu.org/licenses/>.
;*
;*      Licence: GPL-3
;*
;*******************************************************************************

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

;Teensy 1.0: AT90USB162  16 MHz
;Teensy 2.0: ATmega32u4  16 MHz
;AT90USBKEY: AT90USB1287  8 MHz

;Die Mausemulation soll im Hauptprogramm freigeschaltet werden!
;Falls dies nicht erfolgt ist kann es hier geschehen!
.IFNDEF MOUSE
        .EQU MOUSE = 0
.ENDIF
.IFNDEF KEYBOARD
        .EQU KEYBOARD = 0
.ENDIF

;Die Quarzfrequenz soll im Hauptprogramm festgelegt werden.
;Falls dies nicht erfolgt ist kann es hier geschehen!
.IFNDEF CRYSTAL
        .EQU CRYSTAL = 16              ;hier bitte auswaehlen mit 8 bzw. 16
.ENDIF

;IC-Zuweisung, Signatur aus der inc-Datei (Eingebunden im Hauptprogramm)
;ATmega32U4
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x95) && (SIGNATURE_002 == 0x87))
        .EQU AT_MEGA32U4  = 1
.ELSE
        .EQU AT_MEGA32U4 = 0
.ENDIF

;AT90USB162
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x94) && (SIGNATURE_002 == 0x82))
        .EQU AT_90USB162  = 1
.ELSE
        .EQU AT_90USB162  = 0
.ENDIF

;ATmega32U4
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x94) && (SIGNATURE_002 == 0x89))
        .EQU AT_MEGA32U2  = 1
.ELSE
        .EQU AT_MEGA32U2  = 0
.ENDIF

 ;AT90USB1287
.IF ((SIGNATURE_000 == 0x1E) && (SIGNATURE_001 == 0x97) && (SIGNATURE_002 == 0x82))
        .EQU AT_90USB1287  = 1
.ELSE
        .EQU AT_90USB1287  = 0
.ENDIF

;Zuweisung des richtigen Vorteilers fuer die PLL (nicht veraendern!)

.IF AT_MEGA32U4 == 1             ;PPL benoetigt 8 MHz
.IF CRYSTAL == 8
        .EQU PLLPRE = 0x00      ;Bit 2-4 = 0b000 (:1)
.ELSE
        .EQU PLLPRE = 0x10      ;Bit 2-4 = 0b100 (:2)
.ENDIF
.ENDIF  
.IF ((AT_90USB162 == 1)  ||  (AT_MEGA32U2 == 1))   ;PLL benoetigt 8 MHz
.IF CRYSTAL == 8
        .EQU PLLPRE = 0x00      ;Bit 2-4 = 0b000 (:1)
.ELSE
        .EQU PLLPRE = 0x04      ;Bit 2-4 = 0b001 (:2)
.ENDIF
.ENDIF
.IF AT_90USB1287 == 1            ;PLL benoetigt 2 MHz
.IF CRYSTAL == 8
        .EQU PLLPRE = 0x0C      ;Bit 2-4 = 0b011 (:4)
.ELSE
        .EQU PLLPRE = 0x14      ;Bit 2-4 = 0b101 (:8)
.ENDIF
.ENDIF

;Variablen in SRAM

.DSEG                           ;was ab hier folgt kommt in den SRAM-Speicher
;UP INIEP  Parameter
EP_NUM: .BYTE 1                 ;EP NUM: 0-6
EP_TYP: .BYTE 1                 ;EP TYPE: 0=Contr. 1= Iso. 2= Bulk 3 = Inter.
EP_DIR: .BYTE 1                 ;EP DIR: 0=OUT 1=IN
EP_SIZ: .BYTE 1                 ;EP SIZE: 0=8 Byte 1=16 2=32 3=64 (4=128 5=256)
EP_BK:  .BYTE 1                 ;EP BANK: 0=eine Bank 1=doppelte Bank

;Variablen fuer SETUP Paket
SU_R_T:	.BYTE 1 		;bmRequestType
SU_REQ: .BYTE 1 		;bRequest
SU_VAL: .BYTE 2 		;wValue
SU_IND: .BYTE 2 		;wIndex
SU_LEN: .BYTE 2 		;wLength

;Buffer fuer die Endpunkte
EP1_BUF: .BYTE 64               ;Buffer fuer zu sendende Daten (zum PC)
EP2_BUF: .BYTE 64               ;Buffer fuer ankommende Daten (vom PC)
                                ;(erstes Byte beinhaltet Anzahl der Datenbytes)

;Flags und Zaehler fuer die Endpunkte
EP1_FLAG: .BYTE 1               ;Flag Senden zum PC
EP2_FLAG: .BYTE 1               ;Flag Empfang vom PC, wird automatisch gesetzt
                                ;falls Daten vorhanden
EP1_CNT:  .BYTE 1               ;Anzahl der Datenbytes Senden zum PC
EP2_CNT:  .BYTE 1               ;Anzahl der Datenbytes Empfang vom PC

;Konstanten im Flash

.CSEG                           ;was ab hier folgt kommt in den FLASH-Speicher
;Konstanten fuer Endpoints
.EQU    EP0_T = 0               ;EP0 TYPE: 0=Control
.EQU    EP0_D = 0               ;EP0 DIR: 0=OUT
.EQU    EP0_S = 0               ;EP0 SIZE: 0=8 Byte
.EQU    EP0_B = 0               ;EP0 BANK: 0=1 Bank
.EQU    EP0_FS = 8              ;FIFO Size 8 Byte

;EP1 = IN EP (PC <- Device)
.EQU    EP1_T = 3               ;EP1 TYPE: 3=Interrupt
.EQU    EP1_D = 1               ;EP1 DIR: 1=IN
.EQU    EP1_S = 3               ;EP1 SIZE: 3=64 Byte
.EQU    EP1_B = 0               ;EP1 BANK: 0=1 Bank
.EQU    EP1_FS = 64             ;FIFO Size 64 Byte

;EP2 = OUT EP (PC -> Device)
.EQU    EP2_T = 3               ;EP2 TYPE: 3=Interrupt
.EQU    EP2_D = 0               ;EP2 DIR: 0=OUT
.EQU    EP2_S = 3               ;EP2 SIZE: 3=64 Byte
.EQU    EP2_B = 0               ;EP2 BANK: 0=1 Bank
.EQU    EP2_FS = 64             ;FIFO Size 64 Byte

;;; hier erweitern falls weitere EPs benoetigt werden

;weitere globale Konstanten

;bei Erweiterung folgende beiden Konstanten anpassen
.EQU    NR_EPS = 2              ;Benutzte Endpoints (ohne EP0)
;Annahme, dass wTotallength kleiner als 256 Byte ist!
.EQU    WTOTL = 9+9+9+(7*NR_EPS)  ;wTotalLength: Laenge Configuration Deskriptor
                                ; + alle untergeordnete Deskriptoren
                                
.IF MOUSE == 1
        .EQU    HIDDL = 52      ;HID descriptor length
.ELIF KEYBOARD == 1
        .EQU    HIDDL = 59
.ELSE
        .EQU    HIDDL = 32
.ENDIF        
                                
.EQU	LANG_I = 0		;Language Descriptor Index
.EQU	MANU_I = 1		;Manufacturer String Index
.EQU	PROD_I = 2		;Product String Index
.EQU	SERI_I = 3		;Serial Number String Index

;Konstante fuer das Unterprogramm USBINI
;mega32u2 oder '90usb162 besitzen das Bit OTGPADE (enable VBUS Pad) nicht!
.IF ((AT_90USB162 == 1) ||  (AT_MEGA32U2 == 1)) ;WAKEUP Interrupt
        .EQU    USBC1 = 0xA0 ;USBE = 1 FRZCLK = 1 (enable USB)
.ELSE        
        .EQU    USBC1 = 0xB0 ;USBE = 1 FRZCLK = 1 OTGPADE = 1 (enable USB)
.ENDIF        

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Interrupts
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;-------------------------------------------------------------------------------
;       USB GENeral Interrupt Service Routine (reagiert nur auf End Of ReSeT)
;-------------------------------------------------------------------------------
ISRGEN: push    r16             ;rette Register (lokale Variablen und SREG)
        in      r16,SREG
        push    r16

        lds     r16,UDINT       ;falls kein End Of ReSeT Interrupt, dann raus
        sbrs    r16,EORSTI
        rjmp    ISRG1
        lds	r16,UDINT	;cbi UDINT,EORSTI (Bit3)
	andi	r16,0xF7        ;Loesche End Of ReSeT Interrupt Flag
	sts	UDINT,r16

        ;Initialisiere EP0
	ldi     r16,0           ;Parameter fuer UP INITEP
        sts     EP_NUM,r16      ;EP_NUM = 0
        ldi     r16,EP0_T
        sts     EP_TYP,r16
        ldi     r16,EP0_D
        sts     EP_DIR,r16
        ldi     r16,EP0_S
        sts     EP_SIZ,r16
        ldi     r16,EP0_B
        sts     EP_BK,r16
        rcall   INIEP

	lds	r16,UEIENX	;sbi UEIENX,RXSTPE (Bit3)
	ori	r16,0x08        ;Erlaube EP COM Interrupts wenn ein neues
	sts	UEIENX,r16	;gueltiges SETUP Packet vorliegt (RXSTPI=1)

ISRG1:  pop     r16             ;Register wiederherstellen
        out     SREG, r16
        pop     r16
        reti

;-------------------------------------------------------------------------------
;       USB EndPoint COMmunication Interrupt Service Routine
;-------------------------------------------------------------------------------
ISRCOM: push    r16             ;rette Register (lokale Variablen und SREG)
        in      r16,SREG
        push    r16
        
        lds     r16,UEINT       ;EP Nummer durch Bitposition in UEINT
        cpi     r16,0b00000001	;EP0?
        breq    ISREP0
	cpi     r16,0b00000010	;EP1?
        breq    ISREP1
	cpi     r16,0b00000100	;EP2?
        breq    ISREP2

	;;; hier erweitern falls weitere EPs benoetigt werden

	rjmp    ISRCR           ;Raus
	
ISREP0:	ldi	r16,0		;Waehle EP0
	sts     UENUM,r16
        lds     r16,UEINTX      ;falls kein gueltiges SETUP Paket, dann raus
        sbrs    r16,RXSTPI
        rjmp    ISRCR           ;Raus
        rcall	SETUP		;Unterprogramm SETUP (Enumeration)
        rjmp    ISRCR           ;Raus

ISREP1:	ldi	r16,1		;Waehle EP1
	sts     UENUM,r16
        rcall	EP1_IN		;Unterprogramm EP1_IN (IN EP)
        rjmp    ISRCR           ;Raus

ISREP2:	ldi	r16,2		;Waehle EP2
	sts     UENUM,r16
        rcall   EP2_OUT         ;Unterprogramm EP2_OUT (OUT EP)
        rjmp    ISRCR           ;Raus

        ;;; hier erweitern falls weitere EPs benoetigt werden

ISRCR:  pop     r16             ;Register wiederherstellen
        out     SREG, r16
        pop     r16
        reti

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Unterprogramme
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;-------------------------------------------------------------------------------
;       UP INIDEV: USB-Aktivierung (Full-Speed 12Mbit/s) und Interrupts erlauben
;-------------------------------------------------------------------------------
USBINI: push    r16             ;rette Register (lokale Variablen)
.IF AT_MEGA32U4 == 1
        ldi     r16,0x01        ;UHWCON = 0x01, Device Modus und Regelkreis zur
        sts     UHWCON,r16      ;Versorgung der D+ und D- Leitung einschalten
.ENDIF
.IF AT_90USB1287 == 1           
        lds     r16,USBCON      ;FRZCLK=1, Strom sparen
        ori     r16,0x20        
        sts     USBCON,r16
        clr     r16             ;Der ATMEL Bootloader nutzt einige Interrupts
        sts     OTGIEN,r16      ;Deaktiviere diese Interrupts damit sauberer
        sts     UDIEN,r16       ;Start
        sts     UHIEN,r16
        sts     UEIENX,r16
        sts     UPIENX,r16
        ldi     r16,0x80        ;UHWCON = 0x80, Enable Device Modus 
        sts     UHWCON,r16  
        ldi     r16,0x81        ;UHWCON = 0x81, Regelkreis zur
        sts     UHWCON,r16      ;Versorgung der D+ und D- Leitung einschalten
.ENDIF
        ldi     r16,USBC1       ;USBCON = 0xB0 (0xA0), USB Sender/Empf. und Takt
        sts     USBCON,r16      ;einschalten und VBUS-Leitung einschalten
        
        lds     r16,USBCON      ;FRZCLK=0, noetig, da sonst kein WAKEUP Interrupt
        andi    r16,0xDF        ;Bit 5
        sts     USBCON,r16      
        lds     r16,USBCON      ;FRZCLK=1, Strom sparen
        ori     r16,0x20
        sts     USBCON,r16
        
        ;Starte PLL
        ldi     r16,PLLPRE      ;PLL Vorteiler fuer den Quarz (siehe Zuweisungen)
        out     PLLCSR,r16
        ori     r16,0x02        ;PLLE (PLL Enable Bit) = 1, starte PLL
        out     PLLCSR,r16
USBIN1: in      r16,PLLCSR      ;Warte bis PLOCK = 1
        sbrs    r16,PLOCK
        rjmp    USBIN1

        lds     r16,USBCON      ;FRZCLK=0: aktiviere den Takt
        andi    r16,0xDF        ;Bit 5
        sts     USBCON,r16

        clr     r16             ;UDCON = 0 Attach: Verbinde das Device
        sts     UDCON,r16
        ldi     r16,0x08        ;UDIEN = 0x08 (EORSTE = 1)
        sts     UDIEN,r16       ;erlaube End Of ReSeT Interrupt

        ;Endpunkt Buffer, Flags und Zaehler loeschen
        ldi     r16,64
        ldi     XL,LOW(EP1_BUF)
        ldi     XH,HIGH(EP1_BUF)
USBIN2: st      X+,Zero
        dec     r16
        brne    USBIN2
        
        ldi     r16,64
        ldi     XL,LOW(EP2_BUF)
        ldi     XH,HIGH(EP2_BUF)
USBIN3: st      X+,Zero
        dec     r16
        brne    USBIN3
        
        sts     EP1_FLAG,Zero   ;Flag loeschen
        sts     EP2_FLAG,Zero   ;Flag loeschen
        sts     EP1_CNT,Zero    ;Zaehler loeschen
        sts     EP2_CNT,Zero    ;Zaehler loeschen

        sei                     ;Interrupts global erlauben

        pop     r16             ;Register wiederherstellen
        ret
;-------------------------------------------------------------------------------
;       UP INIEP: Aktiviert EP
;       Parameter (jeweils 1 Byte) zur Uebergabe im SRAM (siehe Hauptpr.)
;       EP_NUM 0-6
;       EP_TYP 0=Contr. 1= Iso. 2= Bulk 3 = Inter.
;       EP_DIR 0=OUT 1=IN
;       EP_SIZ 0=8 Byte 1=16 2=32 3=64 (4=128 5=256)
;       EP_BK  0=eine Bank 1=doppelte Bank
;-------------------------------------------------------------------------------
INIEP:  push    r16             ;rette Register (lokale Variablen)
        push    r17

	lds     r16,EP_NUM      ;UENUM = EP_NUM waehle Endpoint
        sts     UENUM,r16
	lds	r16,UECONX	;sbi UECONX,EPEN (Bit 0)
	ori	r16,0x01		;aktiviere EP
	sts	UECONX,r16
        lds     r16,EP_TYP      ;TYPE + DIR festlegen (SF-Reg. UECFG0X)
        ror     r16             ;TYPE in Bit 6 und 7! rotiere 3*rechts durch
        ror     r16             ;carry (kuerzer als 6*links)
        ror     r16
        lds     r17,EP_DIR      ;DIR in Bit 0, andere Bits = 0
        or      r16,r17
        sts     UECFG0X,r16
        lds     r16,EP_SIZ      ;SIZE + BANK festlegen (SF-Reg. UECFG1X)
        lsl     r16             ;SIZ in Bit 4-6 rotiere 4*links
        lsl     r16
        lsl     r16
        lsl     r16
        lds     r17,EP_BK       ;BK in Bit 2-3 rotiere 2*links
        lsl     r17             ;Bit 1 = ALLOC = 0 andere Bits = 0 (reserviert)
        lsl     r17
        or      r16,r17
        sts     UECFG1X,r16
	lds	r16,UECFG1X	;sbi UECFG1X,ALLOC (Bit 1)
	ori	r16,0x02		;Speicher zuweisen
	sts	UECFG1X,r16

	pop     r17             ;Register wiederherstellen
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       UP SETUP: Behandle eine SETUP Anfrage (Paket) vom Host (EP0)
;-------------------------------------------------------------------------------
SETUP:  push    r16             ;rette Register (lokale Variablen)
	push    r17
	push	r18
	push    ZL
	push    ZH

        ;Lese 8 Byte des SETUP Pakets ins SRAM
        lds	r16,UEDATX      ;8 Setup Byte im SRAM ablegen
	sts	SU_R_T,r16	;bmRequestType
	lds	r16,UEDATX
	sts	SU_REQ,r16	;bRequest
	lds	r16,UEDATX
	sts	SU_VAL,r16	;wValue
	lds	r16,UEDATX
	sts	SU_VAL+1,r16
	lds	r16,UEDATX
	sts	SU_IND,r16	;wIndex
	lds	r16,UEDATX
	sts	SU_IND+1,r16
	lds	r16,UEDATX
	sts	SU_LEN,r16	;wLength
	lds	r16,UEDATX
	sts	SU_LEN+1,r16

	lds	r16,UEINTX	;cbi UEINTX,RXSTPI (Bit 3)
	andi	r16,0xF7	;ACK SU Paket (ISR abschl. und FIFO loeschen)
	sts	UEINTX,r16

	lds	r16,SU_R_T	;Kontrolliere ob ein Standard Device Request 
	andi	r16,0x61	;oder eine klassenspezifische Anforderung
	breq	SETUPT	        ;(nur device oder interface; weiter mit SETUPT)    	
	cpi     r16,1;
	breq    SETUPT          
;	cpi	r16,0x20	;(HID Get_Report) vorliegt, sonst STALL (raus)
;	brne	SETUTS
;	rjmp	CLASSR
	rjmp	SETUTS

SETUPT:	lds	r16,SU_REQ	;Teste Art des Requests
	cpi	r16,0x00        ;Get_Status?
	breq	SETUP0		
	cpi	r16,0x05        ;Set_Adress?
	breq	SETUP5		
	cpi	r16,0x06        ;Get_Descriptor?
	breq	SETUP6		
	cpi	r16,0x09        ;Set_Configuration?
	brne	SETUTS		;falls nicht, dann raus (zu weit fuer branch)
	rjmp	SETUP9		
SETUTS:	rjmp	SETUPS		;nicht unterstuetzter Req. -> STALL (und raus)

SETUP0: ;Get_Status
	clr r16
	sts	UEDATX,r16
	sts	UEDATX,r16
	lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;Paket abschicken
	sts	UEINTX,r16
SETU0L:	lds	r16,UEINTX	;Warte bis RXOUTI gesetzt, ZLP (ACK) vom PC
	andi	r16,0x04	;
	breq	SETU0L		;
	rjmp	SETUPR		;raus

SETUP5: ;Set_Adress
	lds	r16,SU_VAL	;LSB(wValue) enthaelt bei Set_Adress Geraeteadr.
	andi	r16,0x7F	;0-127
	sts	UDADDR,r16	;Speichere Adresse
	lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;sende ZLP Pak. (meldet Erfolg, loescht EP Bank)
	sts	UEINTX,r16
SETU51:	lds	r16,UEINTX	;Warte bis TXINI wieder gesetzt (fertig,
	andi	r16,0x01	;ACK vonm PC, EP Bank wieder frei)
	breq	SETU51
	lds	r16,UDADDR	;sbi UDADDR,ADDEN (Bit 7)
	ori	r16,0x80	;Adresse aktivieren
	sts	UDADDR,r16
	rjmp	SETUPR		;raus
	
SETUP6:	;Get_Descriptor
	lds	r16,SU_VAL+1	;HIGH(Su_Val) enthaelt Deskriptor Nummer
	cpi	r16,1		;Device Deskriptor
	breq	SETU6D
        cpi	r16,2		;Configuration Deskriptor
	breq	SETU6C
	cpi	r16,3		;String Deskriptor
	breq	SETU6S
        cpi     r16,34          ;Report Deskriptor (0x22)
        breq    SETU6R
	rjmp	SETUPR		;raus

SETU6D: ;Device Descriptor
        ldi     ZL,LOW(DEVDES*2)
	ldi	ZH,HIGH(DEVDES*2)
	rjmp 	SETU64

SETU6C: ;Configuration Descriptor
        lds	r18,SU_LEN	;Wieviele Bytes angefragt?
        cpi     r18,0           ;Fehler bei NI Visa-Treiber?
        breq    SETU6C1
        lds     r16,SU_LEN+1    ;Manchmal erfragt Windows unsinnig hohe Werte
        cpi     r16,0           ;(groesser als 256 Byte). Dadurch steht dann
        brne    SETU6C1         ;im Lowbyte ev. ein falscher Wert
        cpi     r18,WTOTL       ;Es wird hier davon ausgegangen, dass nicht mehr
        brlo    SETU6C2         ;als 256 Byte angefragt werden koennen.
SETU6C1: ldi    r18,WTOTL         
SETU6C2: ldi	ZL,LOW(CONDES*2)
	ldi	ZH,HIGH(CONDES*2)
	rjmp 	SETU65

SETU6S: lds	r16,SU_VAL	;Enthaelt Sring Index Nummer
	cpi	r16,LANG_I
        breq	SETU60
	cpi	r16,MANU_I	;MANU String Deskriptor
	breq	SETU61
	cpi	r16,PROD_I	;PROD String Deskriptor
	breq	SETU62
	cpi	r16,SERI_I	;SERI String Deskriptor
	breq	SETU63
	rjmp	SETU65
SETU60:	ldi	ZL,LOW(ST0DES*2)
	ldi	ZH,HIGH(ST0DES*2)
	rjmp	SETU64
SETU61:	ldi	ZL,LOW(ST1DES*2)
	ldi	ZH,HIGH(ST1DES*2)
	rjmp	SETU64
SETU62:	ldi	ZL,LOW(ST2DES*2)
	ldi	ZH,HIGH(ST2DES*2)
	rjmp	SETU64
SETU63:	ldi	ZL,LOW(ST3DES*2)
	ldi	ZH,HIGH(ST3DES*2)
	rjmp    SETU64
	
SETU6R: lds     r18,SU_LEN      ;Wieviele Bytes angefragt?
        ldi     ZL,LOW(REPDES*2)
        ldi     ZH,HIGH(REPDES*2)
        rjmp    SETU65	

SETU64:	lpm	r18,Z
SETU65: rcall	SNDDES
	rjmp	SETUPR		;raus

SETUP9:	;Set_Configuration
        ;Benutzte EPs muessen abgeschaltet werden (rueckwaerts, siehe Speicher)
	ldi	r17,NR_EPS	;Zaehler init. um benutzte EPs abzuschalten
SETU9L:	sts	UENUM,r17	;EP auswaehlen
	lds	r16,UECONX	;cbi UECONX,EPEN (Bit 0)
	andi	r16,0xFE	;schalte den EP ab
	sts	UECONX,r16
	lds	r16,UECFG1X	;cbi UECFG1X,ALLOC (Bit 1)
	andi	r16,0xFD	;Speicherbank freigeben
	sts	UECFG1X,r16
	dec	r17
	brne	SETU9L

	;Initialisieren EP1 (IN EP: Parameter und NAKIN Interrupt erlauben)
	ldi	r16,1           ;Parameter fuer UP INITEP
        sts     EP_NUM,r16      ;EP_NUM = 1
        ldi     r16,EP1_T
        sts     EP_TYP,r16
        ldi     r16,EP1_D
        sts     EP_DIR,r16
        ldi     r16,EP1_S
        sts     EP_SIZ,r16
        ldi     r16,EP1_B
        sts     EP_BK,r16
        rcall   INIEP
        lds     r16,UEIENX      ;sbi UEIENX,NAKINE (Bit 6)
        ori     r16,0x40        ;Erlaube NAK IN (EP1) Interrupt (INT wird von HW
        sts     UEIENX,r16      ;gesetzt wenn IN Anfrage von PC mit NAK beantw.
                                ;wurde). Es wird ein neuer COM INT ausgeloest.

	;Initialisieren EP2 (OUT EP: Parameter und RXOUT Interrupt erlauben)
	ldi	r16,2           ;Parameter fuer UP INITEP
        sts     EP_NUM,r16      ;EP_NUM = 2
        ldi     r16,EP2_T
        sts     EP_TYP,r16
        ldi     r16,EP2_D
        sts     EP_DIR,r16
        ldi     r16,EP2_S
        sts     EP_SIZ,r16
        ldi     r16,EP2_B
        sts     EP_BK,r16
        rcall   INIEP
        lds     r16,UEIENX      ;sbi UEIENX,RXOUTE (Bit 2)
        ori     r16,0x04        ;Erlaube RX OUT (EP3) Interrupt (INT wird von HW
        sts     UEIENX,r16      ;gesetzt wenn OUT Daten vorhanden sind).
                                ;Es wird dann ein neuer COM INT ausgeloest.

	;;;Initialisieren EP3
        ;;; hier erweitern falls weitere EPs benoetigt werden

	clr	r16		;EP0 auswaehlen
	sts	UENUM,r16
	lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;sende ZLP Pak. (meldet Erfolg, loescht EP Bank)
	sts	UEINTX,r16
SETU9M:	lds	r16,UEINTX	;Warte bis TXINI wieder gesetzt (fertig,
	andi	r16,0x01	;ACK vonm PC, EP Bank wieder frei)
	breq	SETU9M
	rjmp	SETUPR	
	
CLASSR: lds	r16,SU_REQ	;Teste Art des Klassen Requests
	cpi	r16,0x01        ;Get_Report?
	brne	SETUPS		;nicht unterstuetzter Req. -> STALL (und raus)
	;rcall	GET_RT		;Auskommentiert, da sonst Fehler in Virtualbox
	rjmp	SETUPR	

SETUPS: lds	r16,UECONX	;sbi UECONX,STALLRQ	(Bit 5)
	ori	r16,0x20	;kein Standard Request! erfrage STALL Antwort
	sts	UECONX,r16

SETUPR: pop	ZH
	pop	ZL
	pop	r18
	pop	r17
	pop     r16             ;Register wiederherstellen
        ret

;-------------------------------------------------------------------------------
;       UP WR_DES: Schreibe Deskriptor
;-------------------------------------------------------------------------------
SNDDES:	push	r16		;rette Register (lokale Variablen)
	push	r17

	clr 	r17		;Zaehler1 (geschriebene Bytes) auf 0
SNDDE1: lds	r16,UEINTX	;Unterbrechung vom PC (Host) dann raus
	andi	r16,0x04	;RXOUTI
	brne	SNDDER
	;jeweils nach 8 Bytes das Paket absenden und Speicherbank loeschen
	lpm	r16,Z+		;Byte ins FIFO
	sts	UEDATX,r16
	dec	r18		;Zaehler inkrementieren bzw. dekrementieren
	inc	r17
	ldi	r16,EP0_FS-1	;Ueberpruefe ob FIFO gefuellt (hier 8 Byte)
	and	r16,r17		;Modulo durch UND-Vernuepfung mit 7!
	brne	SNDDE3
	lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;Paket abschicken
	sts	UEINTX,r16
SNDDE2:	lds	r16,UEINTX	;Warte bis TXINI wieder gesetzt (fertig, EP
	andi	r16,0x05	;Bank wieder frei) oder bis RXOUTI gesetzt
	breq	SNDDE2		;(ACK (ZLP) vom PC);
SNDDE3:	cpi	r18,0
	brne	SNDDE1
        ;Kontrolliere ob ACK(ZLP) vom PC angekommen (schon fertig)
        ;(kommt vor wenn Vielfaches des FIFO-Buffers n*8 angefragt waren)
	lds	r16,UEINTX	;Kontrolliere ob ACK (ZLP) vom PC
	andi	r16,0x04  	;angekommen (RXOUTI Flag gesetzt?)
	breq	SNDDE4
	rjmp	SNDDEA		;ACK senden und raus
SNDDE4: lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;Restpaket absenden (loescht EP Bank)
	sts	UEINTX,r16

SNDDE5:	lds	r16,UEINTX	;Warte bis RXOUTI wieder gesetzt
	andi	r16,0x04	;(ZLP (Erfolg) vom PC)
	breq	SNDDE5
SNDDEA:	lds	r16,UEINTX	;cbi UEINTX,RXOUTI (Bit 2)
	andi	r16,0xFB	;Falls ja dann Handshake (ACK), damit Int.
	sts	UEINTX,r16      ;bestaetigt

SNDDER:	pop	r17		;Register wiederherstellen
	pop 	r16
	ret
	
;-------------------------------------------------------------------------------
;       UP GET_RP: HID Request GET_Report Report senden
;-------------------------------------------------------------------------------
GET_RT: push    r16             ;rette Register (lokale Variablen)

        clr r16
	sts	UEDATX,r16
	sts	UEDATX,r16
	lds	r16,UEINTX	;cbi UEINTX,TXINI (Bit 0)
	andi	r16,0xFE	;Paket abschicken
	sts	UEINTX,r16
GET_RL: lds	r16,UEINTX	;Warte bis RXOUTI gesetzt, ZLP (ACK) vom PC
	andi	r16,0x04	;
	breq	GET_RL

	pop     r16             ;Register wiederherstellen
        ret

;-------------------------------------------------------------------------------
;       UP EP1_IN       PC erfragt Daten vom Mikrocontroller  
;-------------------------------------------------------------------------------
EP1_IN: push    r16             ;rette Register (lokale Variablen)
        push    r17
        push    XL
        push    XH
        
        lds     r16,UEINTX      ;Kontrolliere ob Daten vom PC angefragt
        andi    r16,0x40        ;(NAK IN Flag gesetzt?)
        breq    EP1_IR          ;Falls nicht dann raus

        lds     r16,UEINTX      ;cbi UEINTX,NAKINI (Bit 6)
        andi    r16,0xBF        ;NAK_IN Flag soll von Software geloescht werden
        sts     UEINTX,r16

        lds     r16,UEINTX      ;Speicherbank frei?
        andi    r16,0x01        ;(Falls TXINI (fertig, EP Bank wieder frei)
        breq    EP1_IR          ;nicht gesetzt dann raus (siehe 22.14.1 S 275))

        lds     r16,UEINTX      ;cbi UEINTX,TXINI (Bit 0)
        andi    r16,0xFE        ;sende ZLP Pak. (meldet Erfolg, loescht EP Bank)
        sts     UEINTX,r16
        
        lds     r16,EP1_FLAG    ;Sind Daten vorhanden?
        cpi     r16,1
        brne    EP1_I1

        ldi     XL,LOW(EP1_BUF) ;Tabellenzeiger initialisieren
        ldi     XH,HIGH(EP1_BUF)

        lds     r17,EP1_CNT     ;Schleifenzaehler = EP1_CNT
EP1_IL: ld      r16,X+          ;Byte aus dem SRAM-Buffer in die EP1 Buffer
        sts     UEDATX,r16      ;uebertragen
        dec     r17
        brne    EP1_IL

EP1_I1: sts     EP1_FLAG,Zero   ;Flag loeschen

        lds     r16,UEINTX      ;cbi UEINTX,FIFOCON (Bit 7, FIFO Control Bit)
        andi    r16,0x7F        ;Erlaube Abschicken der Daten durch Loeschen
        sts     UEINTX,r16      ;des FIFO Control Bit

EP1_IR: pop     XH              ;Register wiederherstellen
        pop     XL
        pop     r17             
        pop     r16
        ret

;-------------------------------------------------------------------------------
;       UP EP2_OUT      Mikrocontroller erhaelt Daten vom PC
;-------------------------------------------------------------------------------
EP2_OUT: push   r16             ;rette Register (lokale Variablen)
        push    r17
        push    XL
        push    XH

        lds     r16,UEINTX      ;Kontrolliere ob neue Daten (ZLP) vom PC
        andi    r16,0x04        ;(RXOUTI Flag gesetzt?)
        breq    EP2_OR          ;Falls nicht dann raus

        lds     r16,UEINTX      ;cbi UEINTX,RXOUTI (Bit 2)
        andi    r16,0xFB        ;Falls ja dann Handshake (ACK),
        sts     UEINTX,r16      ;damit Interrupt bestaetigt

        ldi     XL,LOW(EP2_BUF+0) ;Tabellenzeiger initialisieren
        ldi     XH,HIGH(EP2_BUF+0)

        ldi     r17,1           ;Schleifenzaehler = 1
EP2_OL: lds     r16,UEINTX      ;Kontrolliere ob Daten fuer Geraet im FIFO
        andi    r16,0x20        ;RWAL (Read/Write Allow Flag) gesetzt?
        breq    EP2_OR          ;Falls nicht dann weiter

        lds     r16,UEDATX      ;Byte einlesen und im SRAM abspeichern
        st      X+,r16
        sts     EP2_CNT,r17     ;Anzahl der eingelesenen Bytes in EP2_CNT
        ldi     r16,1           ;Flag setzen
        sts     EP2_FLAG,r16
        
EP2_O1: inc     r17             ;des SRAM-Buffers
        cpi     r17,65
        brne    EP2_OL

EP2_OR: lds     r16,UEINTX      ;cbi UEINTX,FIFOCON (Bit 7, FIFO Control Bit)
        andi    r16,0x7F        ;Melde, dass Speicherbank wieder frei.
        sts     UEINTX,r16

        pop     XH              ;Register wiederherstellen
        pop     XL
        pop     r17
        pop     r16
        ret

;;; hier erweitern falls weitere EPs benoetigt werden

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Deskriptoren im Programmspeicher (Flash, Word = 2Byte  LSB first!)
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
DEVDES:
;Device Descriptor
.DB     18,0x01         ;bLength = 18 (0x12), Groesse des Deskriptors in Byte
                        ;bDescriptorType = 0x01, Konstante Device = 1
.DB     0x10,0x01       ;bcdUSB = 0x0110, USB_Spec1_1
.DB     0x00,0x00       ;bDeviceClass     = 0x00, Klassencode = 0:
                        ;0 bedeutet: Code definiert auf Interface Level!
                        ;bDeviceSubClass  = 0x00, Unterklassencode = 0 
.DB     0x00,EP0_FS     ;bDeviceProtocoll = 0x00, Protokollcode = 0  
                        ;bMaxPacketSize0 = EP0_FS, max. Paketgr. EP0 (hier 8 B)
.DB     0xeb,0x03       ;idVendor = 0x03eb, Atmel Code durch usb.org vergeben
.DB     0x02,0x00       ;idProduct = 0x0002, Produkt ID (weigu: 2 fuer HID!)
.DB     0x01,0x00       ;bcd_Device hier beliebig
.DB     MANU_I,PROD_I   ;iManufacturer = Index fuer String-Deskriptor Hersteller
                        ;iProduct = Index fuer String-Deskriptor        Produkt
.DB     SERI_I,0x01     ;iSerialNumber = Index fuer String-Deskr. Seriennummer
                        ;bNumConfigurations = 1, Anzahl moeglicher Konfig.
                        
CONDES:
;Configuration Descriptor
.DB     9,0x02          ;bLength = 9, Groesse Deskriptor in Byte
                        ;bDescriptorType = 0x02, Konstante Configuration = 2
.DB     LOW(WTOTL),HIGH(WTOTL) ;wTotalLength: Laenge Configuration Deskriptor
.                       ; + alle untergeordnete Deskriptoren
.DB     0x01,0x01       ;bNumInterfaces = 1
                        ;bConfigurationValue = 1, darf nicht Null sein (sonst
                        ;geht das Geraet in den nicht-konfigurierten Zustand))
.DB     0,0x80          ;iConfiguration = 0, Index fuer String-Deskriptor Conf.
.                       ;(0 = kein Txt)
                        ;bmAttributes = 0x80,bus-powered no remote wakeup Bit7=1
.DB     250,9           ;MaxPower = 250(dezimal), in 2mA Schritten! (500mA)
;Interface Descriptor   ;bLength = 9, Groesse Deskriptor in Byte
.DB     0x04,0          ;bDescriptorType = 0x04, Konstante Interface = 4
                        ;bInterfaceNumber = 0;
.DB     0,NR_EPS        ;bAlternateSetting = 0;
                        ;bNumEndpoints = Anzahl der USB_Endpoints (ohne EP0)
.DB     0x03,0x00       ;bInterfaceClass = 0x03, Klassencode: HID (3)
                        ;bInterfaceSubClass = 0x00, keine Unterklasse (0)
.DB     0x00,0          ;bInterfaceProtocol = 0x00, Protokollcode (0)
                        ;iInterface = 0, Index fuer String-Deskriptor Interface
                        ;(0 = kein Txt)
;HID Descriptor
.DB     0x09,0x21       ;bLength = 0x09, Groesse Deskriptor in Byte
                        ;bDescriptorType = 0x21, verweist auf die HID Klasse
.DB     0x10,0x01       ;bcdHID = 0x0110, HID_Spec 1.1
.DB     0,1             ;bCountryCode = 0 (wird hier nicht unterstuetzt)
                        ;bNumDescriptors = 1 Anzahl der untergeordneten Deskr.
.DB     0x22,LOW(HIDDL) ;bDescriptorType = 0x22, Report Descriptor (obligat.)                        
.DB     HIGH(HIDDL),7   ;wDescriptorLength Laenge des Report Deskriptor
;EP1 Deskriptor         ;bLength = 7, Groesse Deskriptor in Byte
.DB     0x05,0x81       ;bDescriptorType = 0x05, Konstante Endpoint = 5
                        ;bEndpointAddress = 0x81, Bit7=1 (IN) Bit 0-3 EP-Nummer
.DB     0x03,LOW(EP1_FS) ;bmAttributes = 0x03, Transfertyp = Int (3) 
.DB     HIGH(EP1_FS),1  ;(c=0, iso=1, bulk=2) wMaxPacketSize = EP1_FIFO_Size
                        ;bInterval = 1 bei Interrupt: Latenzzeit in Millisek.
;EP2 Deskriptor        
.DB     7,0x05          ;bLength = 7, Groesse Deskriptor in Byte
                        ;bDescriptorType = 0x05, Konstante Endpoint = 5
.DB     0x02,0x03       ;bEndpointAddress = 0x02, Bit 7 = 0 (OUT) Bit 0-3 EP-Nr
                        ;bmAttributes = 0x03, Transfertyp = Int (3) 
.DB     LOW(EP2_FS),HIGH(EP2_FS) ;wMaxPacketSize = EP2_FIFO_Size
.DB     1,0             ;bInterval = 1 bei Interrupt: Latenzzeit in Millisek.
                        ;Fuellbyte da Word (sonst Warnung)

        .;;; hier erweitern falls weitere EPs benoetigt werden
        
ST0DES:
;Language String Deskriptor
.DB     4,0x03          ;bLength = 4, Groesse Deskriptor in Byte
                        ;bDescriptorType = 0x03, Konstante String = 3
.DB     0x09,0x04       ;wLANGID[0] = 0x0409 = English USA (Supp. Lang. Code 0)
ST1DES:
;Manufacturer String Deskriptor
.DB     18,0x03         ;bLength = 18, Groesse Deskriptor in Byte
                        ;bDescriptorType = 0x03, Konstante String = 3
.DB     'W',0,'E',0,'I',0,'G',0,'U',0,'.',0,'L',0,'U',0 ;bString (Unicode 16Bit)

ST2DES:
;Product String Deskriptor
.IF MOUSE == 1   
.DB     20,0x03         ;bLength = 20, Groesse Deskriptor in Byte
                        ;bDescriptorType = 0x03, Konstante String = 3
.DB     'U',0,'S',0,'B',0,'-',0,'M',0,'O',0,'U',0,'S',0,'E',0
.ELIF KEYBOARD == 1

.DB     26,0x03         ;bLength = 22, Groesse Deskriptor in Byte
                        ;bDescriptorType = 0x03, Konstante String = 3
.DB     'H',0,'I',0,'D',0,'-',0,'K',0,'E',0,'Y',0,'B',0,'O',0,'A',0,'R',0,'D',0
.ELSE                                                  ;bString (Unicode 16Bit)
.DB     22,0x03         ;bLength = 22, Groesse Deskriptor in Byte
                        ;bDescriptorType = 0x03, Konstante String = 3
.DB     'H',0,'I',0,'D',0,'-',0,'D',0,'E',0,'V',0,'I',0,'C',0,'E',0
.ENDIF                                                  ;bString (Unicode 16Bit)

ST3DES:
;Serial Number String Deskriptor
.DB     12,0x03         ;bLength = 10, Groesse Deskriptor in Byte
                        ;bDescriptorType = 0x03, Konstante String = 3
.DB     'W',0,'H',0,'1',0,'2',0,'a',0   ;bString = Unicode Encoded String (16Bit)


REPDES:
.IF MOUSE == 1
;Reportdescriptor fuer die HID-Maus
.DB     0x05,0x01       ;USAGE_PAGE (Generic Desktop)
.DB     0x09,0x02       ;USAGE (mouse)
.DB     0xA1,0x01       ;COLLECTION (Application)
.DB     0x09,0x01       ;USAGE (Pointer)
.DB     0xA1,0x00       ;COLLECTION (Physical)
.DB     0x05,0x09       ;USAGE_PAGE (Buttons) Maustasten
.DB     0x19,0x01       ;USAGE_MINIMUM (Button 1) mindestens eine
.DB     0x29,0x03       ;USAGE_MAXIMUM (Button 3) maximal 3
.DB     0x15,0x00       ;LOGICAL_MINIMUM (0)
.DB     0x25,0x01       ;LOGICAL_MAXIMUM (1)
.DB     0x75,0x01       ;REPORT_SIZE (1) Anzahl der Bits pro Datenel.
.DB     0x95,0x03       ;REPORT_COUNT (3) Anzahl der Datenelemente
.DB     0x81,0x02       ;INPUT (Data,Var,Abs) INPUT Bericht, Werte abs.
.DB     0x75,0x05       ;REPORT_SIZE (5) 5 Bits (füllt Byte?)
.DB     0x95,0x01       ;REPORT_COUNT (1) 1*
.DB     0x81,0x01       ;INPUT (Cnst,Var,Abs) Konstantes Feld
.DB     0x05,0x01       ;USAGE_PAGE (Generic Desktop) Mauszeiger
.DB     0x09,0x30       ;USAGE (X) 48
.DB     0x09,0x31       ;USAGE (Y) 49
.DB     0x09,0x38       ;USAGE (Wheel) 56
.DB     0x15,0x81       ;LOGICAL_MINIMUM (-127) -127 bis 127
.DB     0x25,0x7F       ;LOGICAL_MAXIMUM (127)
.DB     0x75,0x08       ;REPORT_SIZE (8) 8 Bits
.DB     0x95,0x03       ;REPORT_COUNT (3) 3*8 Bits (X,Y,Wheel)
.DB     0x81,0x06       ;INPUT (Data,Var,Rel) Werte relativ!
.DB     0xC0,0xC0       ;2* END_COLLECTION

.ELIF KEYBOARD == 1
;Reportdescriptor fuer die HID-Tastatur
.DB     0x05,0x01       ;Usage_Page (Generic Desktop)
.DB     0x09,0x06       ;Usage (Keyboard)
.DB     0xA1,0x01       ;Collection (Application)
.DB     0x05,0x07       ;Usage page (Key Codes)
.DB     0x19,0xE0       ;Usage_Minimum (224)
.DB     0x29,0xE7       ;Usage_Maximum (231)
.DB     0x15,0x00       ;Logical_Minimum (0)
.DB     0x25,0x01       ;Logical_Maximum (1)
.DB     0x75,0x01       ;Report_Size (1)
.DB     0x95,0x08       ;Report_Count (8)
.DB     0x81,0x02       ;Input (Data,Var,Abs) = Modifier Byte
.DB     0x81,0x01       ;Input (Constant) = Reserved Byte
.DB     0x19,0x00       ;Usage_Minimum (0)
.DB     0x29,0x65       ;Usage_Maximum (101)
.DB     0x15,0x00       ;Logical_Minimum (0)
.DB     0x25,0x65       ;Logical_Maximum (101)
.DB     0x75,0x08       ;Report_Size (8)
.DB     0x95,0x06       ;Report_Count (6)
.DB     0x81,0x00       ;Input (Data,Array) = Keycode Bytes(6)
.DB     0x05,0x08       ;Usage Page (LEDs)
.DB     0x19,0x01       ;Usage_Minimum (1)
.DB     0x29,0x05       ;Usage_Maximum (5)
.DB     0x15,0x00       ;Logical_Minimum (0)
.DB     0x25,0x01       ;Logical_Maximum (1)
.DB     0x75,0x01       ;Report_Size (1)
.DB     0x95,0x05       ;Report_Count (5)
.DB     0x91,0x02       ;Output (Data,Var,Abs) = LEDs (5 bits)
.DB     0x95,0x03       ;Report_Count (3)
.DB     0x91,0x01       ;Output (Constant) = Pad (3 bits)
.DB     0xC0,0          ;End_Collection + Fuellbyte

.ELSE
;Reportdescriptor fuer das HID-Geraet
.DB     0x06,0xA0,0xFF,0x09,0x01,0xA1,0x01,0x09,0x03,0x15,0x00,0x25
.DB     0x40,0x95,0x02,0x75,0x08,0x81,0x02,0x09,0x04,0x15,0x00,0x25
.DB     0x40,0x95,0x02,0x75,0x08,0x91,0x02,0xC0
         
         ;0x06,0xA0,0xFF ;USAGE PAGE = 0xFF00 (Vendor Defined Page 1)
         ;0x09,0x01     ;USAGE (Vendor Usage 1)
         ;              ;Funktion des Berichts in der Usage Page
         ;0xA1,0x01     ;COLLECTION (Application) Beginn einer Gruppe
         ;	        ;von Eintraegen
         ;0x09,0x03     ;USAGE (Vendor Usage 1)
         ;0x15,0x00     ;LOGICAL_MINIMUM (1) Legen Bereich Werte fest, die
         ;0x25,0x40     ;LOGICAL_MAXIMUM (64) im Bericht enthalten sind (bin.)
         ;0x95,0x02     ;REPORT_COUNT (64) Anzahl der Datenelemente im Bericht
         ;0x75,0x08     ;REPORT_SIZE (8) Anzahl der Bits pro Datenelement 8Bit
         ;              ;Make sixty-four 8-bit fields (the next time the
         ;              ;parser hits an "Input", "Output", or "Feature" item)
         ;0x81,0x02     ;INPUT (Data,Var,Abs) INPUT Bericht
         ;              ;Instantiates input packet fields based on the above 
         ;              ;report size, count, logical min/max, and usage.
         ;0x09,0x04     ;USAGE (Vendor Usage 1)         		    
         ;0x15,0x00     ;LOGICAL_MINIMUM (1) Legen Bereich Werte fest, die
         ;0x25,0x40     ;LOGICAL_MAXIMUM (64) im Bericht enthalten sind (bin.)
         ;0x95,0x02     ;REPORT_COUNT (64) Anzahl der Datenelemente im Bericht
         ;0x75,0x08     ;REPORT_SIZE (8) Anzahl der Bits pro Datenelement 8Bit
         ;0x91,0x02     ;OUTPUT (Data,Var,Abs) OUTPUT Bericht
         ;              ;Instantiates output packet fields.  Uses same report
         ;              ;size and count as "Input" fields, since nothing
         ;              ;new/different was specified to the parser since the
         ;              ;"Input" item.	
         ;0xC0	        ;END_COLLECTION            
.ENDIF


















