' main source for using the small usb library using hid class
' Firmware zum Testen der HID Klasse der kleinen USB-Bibliothek
' (usb_lib_hid_v1_2.bas)
'
' Datum:  18/02/13                Version: 1.2 (01/06/13)
'
' Prozessor: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u2
'
' Quarz: 8 oder 16 MHz  (siehe Zuweisungen)
'
' Informationen zur Beschaltung:
'      Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):
'
'      Eingaenge:   ADC0 (PF0) analog
'      Ausgaenge:   PD6 (LED on board) und PB0-PB3 LEDS
'
'      AT90USBKEY (AT90USB1287, 8 MHz, www.atmel.com/tools/AT90USBKEY.aspx):
'
'      Eingaenge:   Temperatursensor an ADC0 (PF0) analog
'      Ausgaenge:   PD4-7 (LEDs on board, PD6 blinkt)
'
'      Teensy 1.0 (AT90USB162, 16 MHz, www.pjrc.com/teensy):
'
'      Eingaenge:   PC4
'      Ausgaenge:   PD6 (onboard) und PB0-PB3 LEDS
'
'      Informationen zur Funktionsweise:
'
'      Hauptprogramm zur Nutzung der HID-Bibliothek
'      Ein Timerinterrupt eine LED an PD6 mit 1 Hz blinken.
'      Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
'      abgewickelt. Alle USB-Routinen und Unterprogramme befinden sich in der
'      USB-Bibliothek.
'
'      Fuer beide Endpunkte steht ein 64-Byte grosser Buffer zur Verfuegung.
'
'      Beim Endpunkt 1 (IN) fragt der PC Daten an.
'      Durch Setzen eines Flags (EP1_FLAG) wird dem Endpunkt mitgeteilt, wenn
'      neue Daten vorliegen. Die Anzahl der vorhandenen Bytes steht in der
'      Variablen EP1_CNT. Die Daten selbst werden im Buffer (EP1_BUF) abgelegt.
'
'      Ueber den Endpunkt 2 (OUT) sendet der PC Daten.
'      Die Hauptschleife erkennt an einem Flag (EP2_FLAG) wenn Daten vorliegen.
'      Die Daten liegen im Buffer (EP2_BUF). Die Variable EP2_CNT gibt an wie
'      viele Bytes gesendet wurden.
'
'      Ueber Endpunkt 1 wird dem PC der Wert des AD-Wandlers (ADC0, 2 Byte))
'      uebermittelt.
'      Endpunkt 2 liefert 1 Byte, welche die Information enthaelt, welche LEDs
'      eingeschaltet sind. Bit 6 schaltet das Blinken ein- bzw. aus (Blinkflag)
'
'      Weitere Dokumentation auf www.weigu.lu/b/usb
'
'
' Copyright (c) 2013   Jean-Claude FELTES & Guy WEILER
'                                           weigu[at]weigu[dot]lu

' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.

' Licence: GPL-3


'Hier bitte Chip ($regfile) und Quarz ($crystal) durch Auskommentieren
'fr das entsprechende Board auswaehlen!!
'Teensy 2.0: ATmega32u4  16 MHz
'AT90USBKEY: AT90USB1287  8 MHz
'Teensy 1.0: AT90USB162  16 MHz

$regfile = "m32u4def.dat"               '_CHIP = 75
'$regfile = "usb1287.dat"                '_CHIP = 70
'$regfile = "usb162.dat"                 '_CHIP = 74
'Achtung B2.0.7.3 Fehler in Datei USB_EP statt USB_COM!!
'$regfile = "m32u2def.dat"               '_CHIP = 78
$crystal = 16000000
'$crystal = 8000000

$hwstack = 100
$swstack = 100
$framesize = 100

'Interrupts
On Usb_gen Usb_isr_general
On Usb_com Usb_isr_communication
On Oc1a Isrtc1

' Hier kann wahlweise die Computermaus- oder die Tastatur-Emulation mit 1
' freigeschaltet werden
Const Mouse = 0                         'Mouse = 0 keine Mausemulation
Const Keyboard = 0                      'Keyboard = 0 keine Tastaturemulation

Goto Init                               'Leider noetig damit globale Variablen eingebunden werden
                                         'ohne dass der Code der Isr ausgefhrt wird
$include "/lib/usb_lib_hid_v1_2.bas"    'USB-Bibliothek einbinden
Init:

Dim Blinkflag As Bit                    'Blinken ein/aus
Dim Btmp1 As Byte                       'Zwischenregister
Dim Adcval As Word


#if _chip = 70                           'At90usb1287
Config Pind.4 = Output                  'Ausgang (LED Rot)
Config Pind.5 = Output                  'Ausgang (LED Gruen)
Config Pind.6 = Output                  'Ausgang (LED Gruen)
Config Pind.7 = Output                  'Ausgang (LED Rot)
Led1 Alias Portd.4
Led2 Alias Portd.5
Led3 Alias Portd.6
Led4 Alias Portd.7
#else
Config PinC.7 = Output                  'LEDs Arduino Micro
Config Pinb.0 = Output
Config PinD.5 = Output
Led1 Alias Portb.0
Led2 Alias PortD.5
#endif

Config Clockdiv = 1

'Config Timer1 = Timer
Tccr1a = 0
Tccr1b = &H0D
#if _chip = 70                          'At90usb1287
Ocr1a = 3905
#else
Ocr1a = 7811
#endif
Enable Oc1a                             'Timer Interrupt erlauben

#if _chip = 74                          'AT90usb162
Config Pinc.4 = Input
Portc.4 = 1                             'pull Up
#else
Config Adc = Free , Prescaler = Auto , Reference = Avcc
#endif

Blinkflag = 1                           'Blinken ein

Gosub Usb_init_device                   'USB-Aktivierung

'-------------------------------------------------------------------------------

Do



    Ep1_cnt = 2                         'ADC liefert 2 Byte!
    Disable Interrupts
#if _chip = 74                          'AT90USB162 besitzt keinen A/D-Wandler
    If Pinc.4 = 0 Then                  'Es wird deshalb PC4 als Eingang eingelesen
        Ep1_buf(1) = 0
        Ep1_buf(2) = 0
    Else
        Ep1_buf(1) = Low(1023)
        Ep1_buf(2) = High(1023)
    End If
#else
    Adcval = Getadc(0)                  'ADC Wandlung starten (ADC0)
    Ep1_buf(1) = Low(Adcval)
    Ep1_buf(2) = High(Adcval)
#endif
    Enable Interrupts
    Ep1_flag = 1                        'Flag dass Daten vorhanden setzen

     If Ep2_flag = 1 Then                'neue Daten vorhanden?
         Btmp1 = Ep2_buf(1)
         Blinkflag = Btmp1.6
         Led1 = Btmp1.0
         Led2 = Btmp1.1
         Ep2_flag = 0                    'Flag loeschen
     End If

     If Blinkflag = 1 Then
         Enable Oc1a                    'Timer Interrupt erlauben
     Else
         Disable Oc1a                   'Timer Interrupt sperren
     End If

     waitms 1

Loop

'Interuptroutine Timer1 Compare Match A Interrupt
'Toggelt LED an PD6 / PC7
Isrtc1:
Toggle PortC.7
Return