' main source for using the small usb library using hid class (mouse clone)
' Firmware zum Testen der HID Klasse der kleinen USB-Bibliothek
' Maus-Emulation
'
' Datum:  18/02/13                Version: 1.2 (01/06/13)
'
'       Controller: AT90USB162, AT90USB1287, ATmega32u4, ATmega32u2
'                   (siehe Zuweisungen makefile)
'       Quarz:     8 oder 16 MHz  (siehe Zuweisungen makefile)
'
'       Teensy 2.0 (ATmega32u4, 16 MHz, www.pjrc.com/teensy):
'
'       Eingaenge:   PB7 (Joysticktaste), PB0 (Joystick Links)
'                    PB1 (Joystick Rauf), PB2 (Joystick Rechts)
'                    PB3 (Joystick Runter)
'       Ausgaenge:   PD6 (LED on board) und PB0-PB3 LEDS
'
'       AT90USBKEY (AT90USB1287, 8 MHz, www.atmel.com/tools/AT90USBKEY.aspx):
'
'       Eingaenge:   PB5 (Joysticktaste), PB6 (Joystick Links)
'                    PB7 (Joystick Rauf), PE4 (Joystick Rechts)
'                    PE5 (Joystick Runter)
'       Ausgaenge:   PD4-7 (LEDs on board, PD6 blinkt)
'
'       Teensy 1.0 (AT90USB162, 16 MHz, www.pjrc.com/teensy):
'
'       Eingaenge:   PB7 (Joysticktaste), PB0 (Joystick Links)
'                    PB1 (Joystick Rauf), PB2 (Joystick Rechts)
'                    PB3 (Joystick Runter)
'       Ausgaenge:   PD6 (onboard) und PB0-PB3 LEDS
'
'       Informationen zur Funktionsweise:
'
'       Hauptprogramm zur Nutzung der HID-Bibliothek
'       Ein Timerinterrupt eine LED an PD6 mit 1 Hz blinken.
'       Die gesamte Arbeit zum USB-Bus wird mittels Interrupt-Service-Routinen
'       abgewickelt. Alle USB-Routinen und Unterprogramme befinden sich in der
'       USB-Bibliothek.
'
'       Fuer beide Endpunkte steht ein 64-Byte großer Buffer zur Verfuegung.
'
'       Beim Endpunkt 1 (IN) fragt der PC Daten an.
'       Durch Setzen eines Flags (EP1_FLAG) wird dem Endpunkt mitgeteilt, wenn
'       neue Daten vorliegen. Die Anzahl der vorhandenen Bytes steht in der
'       Variablen EP1_CNT. Die Daten selbst werden im Buffer (EP1_BUF) abgelegt.
'
'       Ueber den Endpunkt 2 (OUT) sendet der PC Daten.
'       Die Hauptschleife erkennt an einem Flag (EP2_FLAG) wenn Daten vorliegen.
'       Die Daten liegen im Buffer (EP2_BUF). Die Variable EP2_CNT gibt an wie
'       viele Bytes gesendet wurden.
'
'       Weiter unten ist die Konstanten Mouse definiert.
'       Mit ihr kann ausgewaehlt werden ob eine Maus emuliert werden soll
'       (Mouse = 1).
'
'       Weitere Dokumentation auf www.weigu.lu/b/usb
'
'
' Copyright (c) 2013   Jean-Claude FELTES & Guy WEILER
'                                           weigu[at]weigu[dot]lu

' This program is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation, either version 3 of the License, or
' (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program.  If not, see <http://www.gnu.org/licenses/>.

' Licence: GPL-3
'Hier bitte Chip ($regfile) und Quarz ($crystal) durch Auskommentieren
'fr das entsprechende Board auswaehlen!!
'Teensy 2.0: ATmega32u4  16 MHz
'AT90USBKEY: AT90USB1287  8 MHz
'Teensy 1.0: AT90USB162  16 MHz

$regfile = "m32u4def.dat"               '_CHIP = 75
'$regfile = "usb1287.dat"                '_CHIP = 70
'$regfile = "usb162.dat"                 '_CHIP = 74
'Achtung B2.0.7.3 Fehler in Datei USB_EP statt USB_COM!!
'$regfile = "m32u2def.dat"               '_CHIP = 78
$crystal = 16000000
'$crystal = 8000000

$hwstack = 100
$swstack = 100
$framesize = 100

'Interrupts
On Usb_gen Usb_isr_general
On Usb_com Usb_isr_communication
On Oc1a Isrtc1

' Hier kann wahlweise die Computermaus- oder die Tastatur-Emulation mit 1
' freigeschaltet werden
Const Mouse = 1                         'Mouse = 0 keine Mausemulation
Const Keyboard = 0                      'Keyboard = 0 keine Tastaturemulation

Goto Init                               'Leider noetig damit globale Variablen eingebunden werden
                                         'ohne dass der Code der Isr ausgefhrt wird
$include "/lib/usb_lib_hid_v1_2.bas"    'USB-Bibliothek einbinden
Init:

Dim Blinkflag As Bit                    'Blinken ein/aus
Dim Btmp1 As Byte                       'Zwischenregister
Dim Adcval As Word


#if _chip = 70                          'At90usb1287

Config Pind.4 = Output                  'Ausgang (LED Rot)
Config Pind.5 = Output                  'Ausgang (LED Gruen)
Config Pind.6 = Output                  'Ausgang (LED Gruen)
Config Pind.7 = Output                  'Ausgang (LED Rot)
Config Pinb.5 = Input                   'Eingang (Joysticktaste (Select))
Config Pinb.6 = Input                   'Eingang (Joystick Links)
Config Pinb.7 = Input                   'Eingang (Joystick Rauf)
Config Pine.4 = Input                   'Eingang (Joystick Rechts)
Config Pine.5 = Input                   'Eingang (Joystick Runter)
Portb.5 = 1                             'Pull-Ups
Portb.6 = 1
Portb.7 = 1
Porte.4 = 1
Porte.5 = 1
Butt1 Alias Pinb.5                      'Linke Maustaste
X_left Alias Pinb.6                     'Linkse       'Links
Y_left Alias Pinb.7                     'Rauf
X_right Alias Pine.4                    'Rechts
Y_right Alias Pine.5                    'Runter

#else

Config Pind.6 = Output                  'LED
Config Pinb.7 = Input                   'Eingang (Joysticktaste (Select))
Config Pinb.0 = Input                   'Eingang (Joystick Links)
Config Pinb.1 = Input                   'Eingang (Joystick Rauf)
Config Pinb.2 = Input                   'Eingang (Joystick Rechts)
Config Pinb.3 = Input                   'Eingang (Joystick Runter)
Portb.7 = 1                             'Pull-Ups
Portb.0 = 1
Portb.1 = 1
Portb.2 = 1
Portb.3 = 1
Butt1 Alias Pinb.7                      'Linke Maustaste
X_left Alias Pinb.0                     'Links
Y_left Alias Pinb.1                     'Rauf
X_right Alias Pinb.2                    'Rechts
Y_right Alias Pinb.3                    'Runter
#endif

Config Clockdiv = 1

'Config Timer1 = Timer
Tccr1a = 0
Tccr1b = &H0D
#if _chip = 70                          'At90usb1287
Ocr1a = 3905
#else
Ocr1a = 7811
#endif
Enable Oc1a                             'Timer Interrupt erlauben

Blinkflag = 1                           'Blinken ein

Gosub Usb_init_device                   'USB-Aktivierung

'-------------------------------------------------------------------------------

Do
  Ep1_cnt = 4                           'Maus liefert 4 Byte!
  'Linke Maustaste
  If Butt1 = 0 Then
     Ep1_buf(1) = 1                     'Falls Button Gedrueckt(nullak.)
  Else                                  'dann setze Butt1 (Linke Maustaste)
     Ep1_buf(1) = 0
  End If
  'X-Koordinate
  If X_right = 0 Then
     Ep1_buf(2) = 1                     'Falls Rechts gedrueckt X = X+1
  Elseif X_left = 0 Then
      Ep1_buf(2) = &HFF                 'Falls Links X = X-1 (Zweierkomplement)
  Else
      Ep1_buf(2) = 0
  End If
  'Y-Koordinate
  If Y_right = 0 Then
    Ep1_buf(3) = 1                      'Falls Runter gedrueckt Y = Y+1
  Elseif Y_left = 0 Then
    Ep1_buf(3) = &HFF                   'Falls Rauf Y = Y -1
  Else
    Ep1_buf(3) = 0
  End If
 'Scrollrad
  Ep1_buf(4) = 0
  Ep1_flag = 1                          'Flag dass Daten vorhanden setzen
Loop

'Interuptroutine Timer1 Compare Match A Interrupt
'Toggelt LED an PD6
Isrtc1:
Toggle Portd.6
Return